#!/bin/sh
set -e    # exit on error (except commands bracketed with "set +e ... set -e")
#
# Script to clean out a cluster directory, removing the files and directories
# added by the CHAP pipeline to restore it back to the original input.  Any
# material added by users to pipeline-created directories will be wiped out,
# but other user files will be left alone.
#

SEQ_DIR=seq.d
EXON_DIR=annot.d
PARAM_FILE=./gmaj.param
PREFS_FILE=./gmaj.prefs
TEMP=temp.d
SCRIPT=`echo $0 | sed -e 's;.*/;;'`    # actual name used, with path removed

if [ $# -ne 1 ]; then
	echo "Usage:  $SCRIPT cluster_directory"
	exit 1
fi
cd $1

#--- from pipeline ---
rm -rf $TEMP
rm -f all.remove_redundancy.gc
rm -f all_1.gc
rm -f all.gc.tmp
rm -f species_tree_with_index.txt
rm -f conversion_in_tree.txt
rm -f non-redundant.gc
rm -f all.gc
if [ -d $SEQ_DIR ]; then
	for sp in `ls $SEQ_DIR`; do
		rm -rf $sp.d
	done
fi
rm -rf ortho.d
rm -rf rm_out.d
rm -rf new_seq.d

#--- from Gmaj ---
rm -f $PARAM_FILE
if [ -f $PREFS_FILE ]; then
	rm -i $PREFS_FILE
fi
if [ -d $EXON_DIR ]; then
	for uf in `ls -1 $EXON_DIR | grep '\.underlays$'`; do
		rm -i $EXON_DIR/$uf
	done
fi
if [ -h chained.maf ]; then    # symlink
	rm chained.maf
fi
if [ -h many-to-many.maf ]; then    # symlink
	rm many-to-many.maf
fi

